//Phong model fragment shader
#version 330 core

in vec3 worldNormal;
in vec3 worldPos;
in vec2 fragmentUV;
in vec4 viewSpace;
in vec4 projCoords;
in vec4 fragPosLightSpace;

layout (location = 0) out vec4 fragColor;


uniform sampler2D color_texture;

uniform bool project;
uniform sampler2D projected_texture;

uniform sampler2D shadowMap;

uniform vec3 lightPos;
uniform vec3 cameraPos;

uniform int material_shininess;
uniform float material_kd;
uniform float material_ks;


const float ambient = 0.08;
//const float ambient = 0.5;
const vec4 diffuseColor = vec4(0.15, 0.05, 0.5, 1.0);
const vec4 specularColor = vec4(1.0, 1.0, 1.0, 1.0);

const vec4 fogColor = vec4(0.25, 0.25, 0.5, 1.0);
const float FogDensity = 0.1;

uniform bool shadows;

float ShadowCalculation(vec4 fragPosLightSpace)
{
    // perform perspective divide
    vec3 projCoords = fragPosLightSpace.xyz / fragPosLightSpace.w;
    // Transform to [0,1] range
    projCoords = projCoords * 0.5 + 0.5;
    // Get closest depth value from light's perspective (using [0,1] range fragPosLight as coords)
    float closestDepth = texture(shadowMap, projCoords.xy).r; 
    // Get depth of current fragment from light's perspective
    float currentDepth = projCoords.z;
    // Calculate bias (based on depth map resolution and slope)
    vec3 normal = normalize(worldNormal);
    vec3 lightDir = normalize(lightPos - worldPos);
    float bias = max(0.05 * (1.0 - dot(normal, lightDir)), 0.005);
    // Check whether current frag pos is in shadow
    // float shadow = currentDepth - bias > closestDepth  ? 1.0 : 0.0;
    // PCF
    float shadow = 0.0;
    vec2 texelSize = 1.0 / textureSize(shadowMap, 0);
    for(int x = -1; x <= 1; ++x)
    {
        for(int y = -1; y <= 1; ++y)
        {
            float pcfDepth = texture(shadowMap, projCoords.xy + vec2(x, y) * texelSize).r; 
            shadow += currentDepth - bias > pcfDepth  ? 1.0 : 0.0;        
        }    
    }
    shadow /= 9.0;
    
    // Keep the shadow at 0.0 when outside the far_plane region of the light's frustum.
    if(projCoords.z > 1.0)
        shadow = 0.0;
        
    return shadow;
}

void main()
{
	vec4 textureColor = texture(color_texture, fragmentUV);
	//fragColor = vec4((worldNormal + 1)/2, 1.0); // to represent normals as colors
	//fragColor = vec4(textureColor, 1.0); // texture only

	vec3 L = normalize(lightPos - worldPos); // light direction
	vec3 V = normalize(cameraPos - worldPos); // viewing direction
 
	float LdotN = max(0, dot(L,normalize(worldNormal)));
 
	float diffuse = material_kd * LdotN; // diffuse light intensity
	
	vec4 diffuseLight = diffuseColor * diffuse;
 
	float specular = 0.0;

	if(LdotN > 0.0)
	{
		//vec3 R = -normalize(reflect(L, worldNormal));//Reflection
		//specular = material_ks * pow(max(0.0, dot(R, V)), material_shininess);
 
		// Blinn-Phong Lighting Model
		vec3 H = normalize(L + V ); // Halfway Vector
		specular = material_ks * pow(max(0, dot(H, normalize(worldNormal))), material_shininess); // specular light
	}

	//vec3 specularLight = specularColor * specular;
	//vec3 ambientLight = vec3(ambient, ambient, ambient);
 
	//float light = ambient + diffuse + specular;

	float shadow = shadows ? ShadowCalculation(fragPosLightSpace) : 0.0;
	//float shadow = 1.0;
    shadow = min(shadow, 0.75); // reduce shadow strength a little: allow some diffuse/specular light in shadowed regions
        float light = (ambient + (1.0 - shadow) * (diffuse + specular));
	   //float light = shadow;
	   //float light = (ambient + shadow * (diffuse + specular));
	//float light = max(ambient, diffuse + specular);

	//vec3 lightColor = ambientLight + diffuseLight + specularLight + textureColor;

	vec4 textureLightColor = light * textureColor;


 
	if (project) { // add projected texture, if projection mode is turned on
		vec2 finalCoords    = projCoords.st / projCoords.q;
		vec4 vProjTexColor  = texture(projected_texture, finalCoords);

		if (projCoords.q > 0.0 && worldNormal.y > 0.0) { // only show projection on the upper side
			textureLightColor = worldNormal.y*0.1*vProjTexColor + textureLightColor;
		} 
   	}


	// add fog:
	float fogFactor = 0;

	float dist = 0; // distance from the viewer
	dist = length(viewSpace);

 	// calculate linear fog
	fogFactor = (20 - dist)/(20 - 10);
   	fogFactor = clamp(fogFactor, 0.0, 1.0);

	
    
    //FragColor = vec4(lighting, 1.0f);

	fragColor = mix(fogColor, textureLightColor, fogFactor);
}
